import { Metadata } from './Metadata';
import { DefinitionMetadata } from './DefinitionMetadata';
/**
 * Keep all [[DefinitionMetadata]] and [[GenericMetadata]].
 * Provide methods to query metadata with its associated class or graphql type name.
 */
export declare class MetadataStorage {
    readonly genericMetadata: any[];
    readonly definitionMetadata: DefinitionMetadata[];
    /**
     * Add a new [[Metadata]] object to storage.
     * @param metadata A metadata object to register
     */
    register(metadata: Metadata): void;
    /**
     * Get a [[DefinitionMetadata]] object which is instance of the `metadataClass` and associated to `definitionClass`
     * @param metadataClass A [[DefinitionMetadata]] subclass to query
     * @param definitionClass A class associated with metadata to query
     */
    getDefinitionMetadata<T extends DefinitionMetadata>(metadataClass: {
        new (...args: any[]): T;
    }, definitionClass: Function): T;
    /**
     * Get a [[DefinitionMetadata]] object by the name of the GraphQLType that the metadata will generate.
     * @param name The name of a GraphQLType which will be built from metadata to query
     */
    getDefinitionMetadataByName(name: string): DefinitionMetadata;
    /**
     * Find all [[GenericMetadata]] which is instance of the `metadataClass` and associated to `definitionClass`
     * @param metadataClass A [[GenericMetadata]] subclass to query
     * @param definitionClass A class associated with metadata to query
     */
    findGenericMetadata<T extends Metadata>(metadataClass: {
        new (...args: any[]): T;
    }, definitionClass: Function): T[];
}
