"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var execute_1 = require("graphql/execution/execute");
var ArgumentMetadata_1 = require("./ArgumentMetadata");
var GenericMetadata_1 = require("../base/GenericMetadata");
var types_1 = require("../types");
/**
 * Metadata type for fields
 */
var FieldMetadata = /** @class */ (function (_super) {
    __extends(FieldMetadata, _super);
    function FieldMetadata() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Object.defineProperty(FieldMetadata.prototype, "fieldName", {
        get: function () {
            return this.config.fieldName;
        },
        enumerable: true,
        configurable: true
    });
    FieldMetadata.prototype.findArgumentMetadata = function () {
        var _this = this;
        return this.storage
            .findGenericMetadata(ArgumentMetadata_1.ArgumentMetadata, this.definitionClass)
            .filter(function (meta) { return meta.fieldName === _this.config.fieldName; });
    };
    Object.defineProperty(FieldMetadata.prototype, "fieldConfig", {
        get: function () {
            var argumentMetadata = this.findArgumentMetadata();
            var type = this.type;
            var resolve = this.resolve;
            var args = argumentMetadata.reduce(function (results, metadataObj) {
                results[metadataObj.argumentName] = metadataObj.argumentConfig;
                return results;
            }, {});
            var _a = this.config, description = _a.description, deprecationReason = _a.deprecationReason;
            var fieldConfig = { type: type, args: args, description: description, deprecationReason: deprecationReason, resolve: resolve };
            return fieldConfig;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FieldMetadata.prototype, "completeArguments", {
        get: function () {
            var argumentMetadata = this.findArgumentMetadata();
            return function (args, context, info) { return argumentMetadata.reduce(function (completeArgs, meta) {
                completeArgs[meta.argumentName] = meta.instantiate(args[meta.argumentName], context, info);
                return completeArgs;
            }, {}); };
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FieldMetadata.prototype, "resolve", {
        get: function () {
            var definitionClass = this.definitionClass;
            var fieldName = this.config.fieldName;
            var _a = this, completeArguments = _a.completeArguments, instantiate = _a.instantiate;
            var descriptor = Object.getOwnPropertyDescriptor(definitionClass, fieldName);
            var describedFunction = descriptor && (descriptor.value || descriptor.get);
            var resolver = describedFunction instanceof Function
                ? describedFunction
                : undefined;
            return function (source, args, context, info) {
                var completeArgs = completeArguments(args, context, info);
                var resolved;
                if (resolver) {
                    resolved = resolver.apply(definitionClass, [source, completeArgs, context, info]);
                }
                else if (source) {
                    resolved = execute_1.defaultFieldResolver(source, completeArgs, context, info);
                }
                else {
                    return source;
                }
                if (!resolved) {
                    return resolved;
                }
                else if (types_1.isPromise(resolved)) {
                    return resolved.then(instantiate);
                }
                return instantiate(resolved);
            };
        },
        enumerable: true,
        configurable: true
    });
    return FieldMetadata;
}(GenericMetadata_1.GenericMetadata));
exports.FieldMetadata = FieldMetadata;
//# sourceMappingURL=FieldMetadata.js.map