import { GraphQLObjectType, GraphQLFieldConfigMap, GraphQLInterfaceType } from "graphql";
import { DefinitionMetadata, DefinitionMetadataConfig } from "../base/DefinitionMetadata";
import { FieldMetadata } from "./FieldMetadata";
import { Instantiator } from "../types";
import { TypeExpression } from "../type-expression/TypeExpression";
export interface ObjectTypeMetadataConfig extends DefinitionMetadataConfig {
    description?: string;
    interfaces?: TypeExpression[];
}
/**
 * Metadata type for ObjectType
 */
export declare class ObjectTypeMetadata<TConfig extends ObjectTypeMetadataConfig = ObjectTypeMetadataConfig> extends DefinitionMetadata<TConfig> {
    protected getFieldMetadata(): FieldMetadata[];
    fields(): GraphQLFieldConfigMap<any, any>;
    /**
     * Build GraphQLObjectType instance from metadata.
     */
    readonly typeInstance: GraphQLObjectType;
    /**
     * Default source type validator
     * @param source
     */
    isTypeOf(source: any): boolean;
    readonly interfaces: GraphQLInterfaceType[] | undefined;
    /**
     * Get the instantiator function from definition class or return default
     */
    readonly instantiate: Instantiator;
}
