import { DefinitionNode, FieldDefinitionNode, InputObjectTypeDefinitionNode, InputValueDefinitionNode, InterfaceTypeDefinitionNode, ListTypeNode, NamedTypeNode, NonNullTypeNode, ObjectTypeDefinitionNode } from 'graphql';
import { ArgumentMetadataConfig } from '../metadata/ArgumentMetadata';
import { FieldMetadataConfig } from '../metadata/FieldMetadata';
import { InputFieldMetadataConfig } from '../metadata/InputFieldMetadata';
import { InputObjectTypeMetadataConfig } from '../metadata/InputObjectTypeMetadata';
import { ObjectTypeMetadataConfig } from '../metadata/ObjectTypeMetadata';
import { TypeArg, TypeExpression } from '../type-expression/TypeExpression';
import { Lazy } from '../types';
export interface TypeSubstitutionMap {
    [tempName: string]: TypeExpression | TypeArg | Lazy<TypeArg>;
}
export declare function gql(strings: TemplateStringsArray, ...typeArgs: Array<TypeExpression | TypeArg | Lazy<TypeArg>>): ASTParser;
export declare class ASTParser {
    readonly objectTypeMetadataConfigs: ObjectTypeMetadataConfig[];
    readonly interfaceTypeMetadataConfigs: InputObjectTypeMetadataConfig[];
    readonly inputObjectTypeMetadataConfigs: InputObjectTypeMetadataConfig[];
    readonly fieldMetadataConfigs: FieldMetadataConfig[];
    readonly argumentMetadataConfigs: ArgumentMetadataConfig[];
    readonly inputFieldMetadataConfigs: InputFieldMetadataConfig[];
    constructor(rootNode: DefinitionNode, substitutionMap: TypeSubstitutionMap);
    protected rootNode: DefinitionNode;
    protected substitutionMap: TypeSubstitutionMap;
    protected createMetadataFromAST(): void;
    protected completeTypeExpression(type: NamedTypeNode | ListTypeNode | NonNullTypeNode): TypeExpression;
    protected appendObjectTypeConfig(node: ObjectTypeDefinitionNode): void;
    protected appendInterfaceTypeMetadataConfig(node: InterfaceTypeDefinitionNode): void;
    protected appendInputObjectTypeMetadataConfig(node: InputObjectTypeDefinitionNode): void;
    protected appendFieldMetadataConfig(node: FieldDefinitionNode): void;
    protected appendArgumentMetadataConfig(node: InputValueDefinitionNode, fieldName: string): void;
    protected appendInputFieldMetadataConfig(node: InputValueDefinitionNode): void;
}
