"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var globalMetadataStorage_1 = require("../globalMetadataStorage");
var ObjectTypeMetadata_1 = require("../metadata/ObjectTypeMetadata");
var InterfaceTypeMetadata_1 = require("../metadata/InterfaceTypeMetadata");
var InputFieldMetadata_1 = require("../metadata/InputFieldMetadata");
var InputObjectTypeMetadata_1 = require("../metadata/InputObjectTypeMetadata");
var FieldMetadata_1 = require("../metadata/FieldMetadata");
var ArgumentMetadata_1 = require("../metadata/ArgumentMetadata");
/**
 * Register definition metadata and generic metadata from its given parse result
 * @param parseResult
 */
var TypeDefinition = /** @class */ (function () {
    function TypeDefinition(parser, storage) {
        this.storage = storage;
        this.parser = parser;
        this.handleObjectType = this.handleObjectType.bind(this);
        this.handleInterfaceType = this.handleInterfaceType.bind(this);
        this.handleInputObjectType = this.handleInputObjectType.bind(this);
        this.handleField = this.handleField.bind(this);
        this.handleArgument = this.handleArgument.bind(this);
        this.handleInputField = this.handleInputField.bind(this);
    }
    TypeDefinition.prototype.handleObjectType = function (config) {
        (new ObjectTypeMetadata_1.ObjectTypeMetadata(this.definitionClass, config)).registerToStorage(this.storage);
    };
    TypeDefinition.prototype.handleInterfaceType = function (config) {
        (new InterfaceTypeMetadata_1.InterfaceTypeMetadata(this.definitionClass, config)).registerToStorage(this.storage);
    };
    TypeDefinition.prototype.handleInputObjectType = function (config) {
        (new InputObjectTypeMetadata_1.InputObjectTypeMetadata(this.definitionClass, config)).registerToStorage(this.storage);
    };
    TypeDefinition.prototype.handleField = function (config) {
        (new FieldMetadata_1.FieldMetadata(this.definitionClass, config)).registerToStorage(this.storage);
    };
    TypeDefinition.prototype.handleArgument = function (config) {
        (new ArgumentMetadata_1.ArgumentMetadata(this.definitionClass, config)).registerToStorage(this.storage);
    };
    TypeDefinition.prototype.handleInputField = function (config) {
        (new InputFieldMetadata_1.InputFieldMetadata(this.definitionClass, config)).registerToStorage(this.storage);
    };
    TypeDefinition.prototype.decorate = function (definitionClass) {
        this.definitionClass = definitionClass;
        var _a = this.parser, objectTypeMetadataConfigs = _a.objectTypeMetadataConfigs, interfaceTypeMetadataConfigs = _a.interfaceTypeMetadataConfigs, inputObjectTypeMetadataConfigs = _a.inputObjectTypeMetadataConfigs, fieldMetadataConfigs = _a.fieldMetadataConfigs, argumentMetadataConfigs = _a.argumentMetadataConfigs, inputFieldMetadataConfigs = _a.inputFieldMetadataConfigs;
        objectTypeMetadataConfigs.forEach(this.handleObjectType);
        interfaceTypeMetadataConfigs.forEach(this.handleInterfaceType);
        inputObjectTypeMetadataConfigs.forEach(this.handleInputObjectType);
        fieldMetadataConfigs.forEach(this.handleField);
        argumentMetadataConfigs.forEach(this.handleArgument);
        inputFieldMetadataConfigs.forEach(this.handleInputField);
    };
    TypeDefinition.createDecorator = function (parser, storage) {
        var _this = this;
        return function (definitionClass) {
            (new _this(parser, storage || globalMetadataStorage_1.globalMetadataStorage)).decorate(definitionClass);
        };
    };
    return TypeDefinition;
}());
exports.TypeDefinition = TypeDefinition;
var AbstractTypeDefinition = /** @class */ (function (_super) {
    __extends(AbstractTypeDefinition, _super);
    function AbstractTypeDefinition() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    AbstractTypeDefinition.prototype.handleObjectType = function (config) { };
    AbstractTypeDefinition.prototype.handleInterfaceType = function (config) { };
    AbstractTypeDefinition.prototype.handleInputObjectType = function (config) { };
    return AbstractTypeDefinition;
}(TypeDefinition));
exports.AbstractTypeDefinition = AbstractTypeDefinition;
exports.defineType = TypeDefinition.createDecorator.bind(TypeDefinition);
exports.defineAbstractType = AbstractTypeDefinition.createDecorator.bind(AbstractTypeDefinition);
//# sourceMappingURL=definition.js.map