"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var graphql_1 = require("graphql");
var types_1 = require("../types");
var DefinitionMetadata_1 = require("../base/DefinitionMetadata");
/**
 * Contain an argument which can be resolved to GraphQLType instance.
 * Find a instantiator from associated class instance or return default instantiator
 */
var TypeExpression = /** @class */ (function () {
    function TypeExpression(typeArg) {
        this.typeArg = typeArg;
    }
    TypeExpression.prototype.getCompleteTypeArg = function () {
        var typeArg = this.typeArg;
        return types_1.isLazy(typeArg) ? typeArg() : typeArg;
    };
    TypeExpression.prototype.resolveDefinitionMetadata = function (storage) {
        var completeTypeArg = this.getCompleteTypeArg();
        if (completeTypeArg instanceof Function) {
            return storage.getDefinitionMetadata(DefinitionMetadata_1.DefinitionMetadata, completeTypeArg);
        }
        else if (typeof completeTypeArg === 'string') {
            return storage.getDefinitionMetadataByName(completeTypeArg);
        }
        else {
            throw new Error("Cannot find any DefinitionMetadata with TypeExpression of " + completeTypeArg);
        }
    };
    TypeExpression.prototype.buildInstantiator = function (storage) {
        var completeTypeArg = this.getCompleteTypeArg();
        if (graphql_1.isType(completeTypeArg)) {
            return types_1.defaultInstantiator;
        }
        var _a = this.resolveDefinitionMetadata(storage), instantiate = _a.instantiate, definitionClass = _a.definitionClass;
        return function (source, context, info) {
            return source instanceof definitionClass ? source : instantiate(source);
        };
    };
    TypeExpression.prototype.buildTypeInstance = function (storage) {
        var completeTypeArg = this.getCompleteTypeArg();
        if (graphql_1.isType(completeTypeArg)) {
            return completeTypeArg;
        }
        var metadata = this.resolveDefinitionMetadata(storage);
        return metadata.typeInstance;
    };
    return TypeExpression;
}());
exports.TypeExpression = TypeExpression;
//# sourceMappingURL=TypeExpression.js.map